from typing import TYPE_CHECKING

from pydantic import BaseModel

if TYPE_CHECKING:
    from wexample_prompt.responses.abstract_prompt_response import AbstractPromptResponse


class AbsractOutputHandler(BaseModel):
    def print(self, response: "AbstractPromptResponse") -> None:
        ...



import sys
from typing import TYPE_CHECKING

from wexample_prompt.output.abstract_output_handler import AbsractOutputHandler

if TYPE_CHECKING:
    from wexample_prompt.responses.abstract_prompt_response import AbstractPromptResponse


class StdoutOutputHandler(AbsractOutputHandler):
    def print(self, response: "AbstractPromptResponse") -> None:
        print(response.render(), file=sys.stdout, end="\n")



from typing import List

from pydantic import Field

from wexample_prompt.output.abstract_output_handler import AbsractOutputHandler
from wexample_prompt.responses.abstract_prompt_response import AbstractPromptResponse


class BufferOutputHandler(AbsractOutputHandler):
    buffer: List[AbstractPromptResponse] = Field(default_factory=list)

    def print(self, response: AbstractPromptResponse) -> None:
        self.buffer.append(response)



from typing import Optional, TYPE_CHECKING

from wexample_prompt.common.io_manager import IoManager

if TYPE_CHECKING:
    from wexample_prompt.output.abstract_output_handler import AbsractOutputHandler


class WithIoManager:
    _io: Optional[IoManager] = None

    def __init__(self, io: Optional[IoManager] = None) -> None:
        self._io = io

    @property
    def io(self) -> IoManager:
        return self._io

    @io.setter
    def io(self, manager: IoManager) -> None:
        """Set the IoManager instance."""
        self._io = manager

    def _init_io_manager(self, output: Optional["AbsractOutputHandler"] = None) -> None:
        self._io = IoManager(output=output)
